# -*- coding: utf-8 -*-

try:
    import _thread as thread
    print("use python3")
except ImportError:
    import thread
    print("use python2")


def singleton(multi_init=False):
    """decorator for a thread-safe singleton pattern.
        Please just use the singleton pattern if it is not
        possible do solve your problem in another way.

        Example:

        >>> @singleton()
        ... class MySingleton(object):
        ...     pass
        >>> o = MySingleton()
        >>> o2 = MySingleton()
        >>> print o is o2

    """
    def _singleton_decorator(decorated_class):
        class _singleton(decorated_class):
            _instance = None
            _lock = thread.allocate_lock()

            def __new__(cls, *args, **kwargs):
                with cls._lock:
                    if _singleton._instance is None:
                        _singleton._instance = super(_singleton, cls).__new__(cls, *args, **kwargs)
                        _singleton._instance._initialized = False
                return _singleton._instance

            def __init__(self, *args, **kwargs):
                if self._initialized and not multi_init:
                    return
                super(_singleton, self).__init__(*args, **kwargs)
                self._initialized = True

        _singleton.__name__ = decorated_class.__name__
        return _singleton
    return _singleton_decorator
